# Generic Templater

A python module that ingests yaml and jinja templates and spits out templated files. 
Works like Ansible, but without Ansible.


## How does it work like ansible?

- Yaml syntax for supplying of variables
- Hierarchical variable precidence
- Very easy to use
- Meta-templating (source path, destination path, variable manipulation)
- Can be executed remotely (Server with templates, client gets file)


## Components

- [Jinja](https://jinja.palletsprojects.com/en/3.1.x/) for templating
- [Click](https://click.palletsprojects.com/en/8.1.x/) for CLI
- [Paramiko](https://www.paramiko.org/) for remote execution
- [PyYAML](https://pyyaml.org/wiki/PyYAMLDocumentation) for ingesting YAML

## Code in practice (maybe)

```python

from gentem import Templater

template_dir = "/opt/gentem/templates/"
vars_dir = "/opt/gentem/vars/"

# names of matched .yaml files and .j2 in the above directories
items = ["host1", "host2", "host3"]

var_path = "{{ hostname }}.yml"
template_path = "{{ hostname }}.cfg.j2"
destination_path = "/etc/app_to_configure/{{ hostname }}.cfg"

templater = Templater(template_dir, vars_dir)

for i in items:
    # Just generates UTF-8 encoded string. hostname being picked up by the **kwargs definition, as all inline variables are
    templated_file = templater.generate(template_path=template_path, var_path=var_path, hostname=i)

    # Will also read from ssh config files
    templater.send(content=templated_file, path=destination_path, hostname=i, user="root", identity_file="/home/cam/.ssh/id_rsa")

    # Can also write to file locally
```

